{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000
  http://www.zeitungsjunge.de/delphi/ }

unit Main;

interface

uses
 Windows,
 Forms,
 StdCtrls,
 Controls,
 Classes,
 
 rjPcreRegExp;

type
 TForm1 = class (TForm)
  btnMatch: TButton;
  lblResult: TLabel;
  MemoResult: TMemo;
  lblPattern: TLabel;
  lblSubject: TLabel;
  memoSubject: TMemo;
  PcreRegExp: TPcreRegExp;
  cboPatterns: TComboBox;
  lblSelect: TLabel;
  edtPattern: TEdit;
  procedure btnMatchClick (Sender: TObject);
  procedure FormCreate (Sender: TObject);
  procedure cboPatternsChange (Sender: TObject);
 end;
 
var
 Form1              : TForm1;
 
implementation

{$R *.DFM}

uses
 SysUtils;

type
 PatternExample = record
  Name: AnsiString;
  Pattern: AnsiString;
  Subject: AnsiString;
 end;
 
const
 PatternExamples    : array[0..3] of PatternExample = (
  (Name: 'E-Mail Address';
  Pattern: '([_a-zA-Z\d\-\.]+)@([_a-zA-Z\d\-]+)\.([_a-zA-Z\d\-]+)';
  Subject: 'Drop me a line at ralfjunker@gmx.de if you have an idea how to improve rjPcreRegExp.'),

  (Name: 'Internet-URL';
  Pattern: '(FTP|HTTP)://([_a-z\d\-]+(\.[_a-z\d\-]+)+)((/[_a-z\d\-\\\.]+)+)*';
  Subject: 'For updates of rjPcreRegExp visit http://www.zeitungsjunge.de/delphi/ You will also find other Delphi Freeware components there.'),

  (Name: 'IP-Address';
  Pattern: '([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])';
  Subject: 'IP-Addresseslike, i.e. 187.224.98.4, consist of four bytes separated by dots.'),

  (Name: 'Word starting with the letter ''a''';
  Pattern: '\ba.[^ ]*';
  Subject: 'Not all words always start with the letter ''a''. However, rjPcreRegExp will find the first one in this sentence.'));
 
procedure TForm1.FormCreate (Sender: TObject);
var
 i                  : Integer;
begin
 Form1.Caption := 'rjPcreRegExp Demo (PCRE Version: ' + TPcreRegExp.GetVersion + ')';
 
 with cboPatterns do
  begin
   with Items do
    begin
     Clear;
     for i := Low (PatternExamples) to High (PatternExamples) do
      AddObject (PatternExamples[i].Name, Pointer (i));
    end;
   ItemIndex := 0;
   OnChange (nil);
  end;
end;

procedure TForm1.btnMatchClick (Sender: TObject);
var
 i                  : Integer;
 MatchResult        : Integer;
begin
 MemoResult.Lines.Clear;
 
 with PcreRegExp do
  begin
   Pattern := edtPattern.Text;
   SetSubjectString (memoSubject.Text);

   MatchResult := Match;

   if MatchResult >= 0 then
    begin
     MemoResult.Lines.Add ('Success.');
     MemoResult.Lines.Add ('');

     if MatchResult > 0 then
      begin
       MemoResult.Lines.Add ('SubStringCount is ' + IntToStr (MatchResult) + '.');
       MemoResult.Lines.Add ('');
       for i := 0 to SubStringCount - 1 do
        MemoResult.Lines.Add (IntToStr (i) + ' - ' + SubStrings[i]);
      end
     else
      MemoResult.Lines.Add ('No SubStrings Captured.');
    end
   else
    begin
     MemoResult.Lines.Add ('No match.');
     MemoResult.Lines.Add (Error);
    end;

  end;
 
end;

procedure TForm1.cboPatternsChange (Sender: TObject);
begin
 with PatternExamples[cboPatterns.ItemIndex] do
  begin
   edtPattern.Text := Pattern;
   memoSubject.Text := Subject;
  end;
end;

end.

