{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000
  http://www.zeitungsjunge.de/delphi/ }

unit rjCHelpers;



interface

{ **************************************************************************** }
{ Delphi styled C helper functions
{ **************************************************************************** }

{ Memory Allocation }

function _malloc (Size: Cardinal): Pointer; cdecl;
procedure _free (p: Pointer); cdecl;

{ Memory Operation }

procedure _memset (Src: Pointer; c: Integer; N: Cardinal); cdecl;
procedure _memmove (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
procedure _memcpy (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;

{ Characters & Strings }

function _isalnum (c: AnsiChar): Boolean; cdecl;
function _isalpha (c: AnsiChar): Boolean; cdecl;
function _iscntrl (c: AnsiChar): Boolean; cdecl;
function _isdigit (c: AnsiChar): Boolean; cdecl;
function _isgraph (c: AnsiChar): Boolean; cdecl;
function _islower (c: AnsiChar): Boolean; cdecl;
function _isprint (c: AnsiChar): Boolean; cdecl;
function _ispunct (c: AnsiChar): Boolean; cdecl;
function _isspace (c: AnsiChar): Boolean; cdecl;
function _isupper (c: AnsiChar): Boolean; cdecl;
function _isxdigit (c: AnsiChar): Boolean; cdecl;
function __ltolower (c: AnsiChar): AnsiChar; cdecl;
function __ltoupper (c: AnsiChar): AnsiChar; cdecl;
function _StrChr (const s: PAnsiChar; c: AnsiChar): PAnsiChar; cdecl;
function _StrNCmp (const s1, s2: PAnsiChar; MaxLen: Cardinal): Integer; cdecl;

{ **************************************************************************** }
{ **************************************************************************** }

function pcre_malloc (Size: Cardinal): Pointer; cdecl;
procedure pcre_free (p: Pointer); cdecl;

procedure memset (Src: Pointer; c: Integer; N: Cardinal); cdecl;
procedure memmove (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
procedure memcpy (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;

function StrChr (const s: PAnsiChar; c: AnsiChar): PAnsiChar; cdecl;
function StrNCmp (const s1, s2: PAnsiChar; MaxLen: Cardinal): Integer; cdecl;

implementation

uses
 Windows,
 SysUtils;

{ Memory Allocation }

function _malloc (Size: Cardinal): Pointer; cdecl;
begin
 GetMem (Result, Size);
end;

procedure _free (p: Pointer); cdecl;
begin
 FreeMem (p);
end;

{ Memory Operation }

procedure _memset (Src: Pointer; c: Integer; N: Cardinal); cdecl;
begin
 FillChar (Src^, N, c);
end;

procedure _memmove (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
begin
 Move (Src^, Dest^, N);
end;

procedure _memcpy (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
begin
 Move (Src^, Dest^, N);
end;

{ Characters & Strings }

function _isalnum (c: AnsiChar): Boolean; cdecl;
begin
 Result := IsCharAlphaNumeric (c);
end;

function _isalpha (c: AnsiChar): Boolean; cdecl;
begin
 Result := IsCharAlpha (c);
end;

function _iscntrl (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := (CharType and C1_CNTRL) <> 0;
end;

function _isdigit (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := Boolean (CharType and C1_DIGIT);
end;

function _isgraph (c: AnsiChar): Boolean; cdecl;
begin
 Result := IsCharAlphaNumeric (c);
end;

function _islower (c: AnsiChar): Boolean; cdecl;
begin
 Result := IsCharLower (c);
end;

function _isprint (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := (CharType and C1_CNTRL) = 0;
end;

function _ispunct (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := (CharType and C1_PUNCT) <> 0;
end;

function _isspace (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := (CharType and C1_BLANK) <> 0;
end;

function _isupper (c: AnsiChar): Boolean; cdecl;
begin
 Result := IsCharUpper (c);
end;

function _isxdigit (c: AnsiChar): Boolean; cdecl;
var
 CharType           : Word;
begin
 GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, @c, SizeOf (c), CharType);
 Result := (CharType and C1_XDIGIT) <> 0;
end;

function __ltolower (c: AnsiChar): AnsiChar; cdecl;
begin
 CharLower (@c);
 Result := c;
end;

function __ltoupper (c: AnsiChar): AnsiChar; cdecl;
begin
 CharUpper (@c);
 Result := c;
end;

function _StrChr (const s: PAnsiChar; c: AnsiChar): PAnsiChar; cdecl;
begin
 Result := StrScan (s, c);
end;

function _StrNCmp (const s1, s2: PAnsiChar; MaxLen: Cardinal): Integer; cdecl;
begin
 Result := StrLComp (s1, s2, MaxLen);
end;

{ **************************************************************************** }

function pcre_malloc (Size: Cardinal): Pointer; cdecl;
begin
 GetMem (Result, Size);
end;

procedure pcre_free (p: Pointer); cdecl;
begin
 FreeMem (p);
end;

procedure memset (Src: Pointer; c: Integer; N: Cardinal); cdecl;
begin
 FillChar (Src^, N, c);
end;

procedure memmove (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
begin
 Move (Src^, Dest^, N);
end;

procedure memcpy (Dest: Pointer; Src: Pointer; N: Cardinal); cdecl;
begin
 Move (Src^, Dest^, N);
end;

function StrChr (const s: PAnsiChar; c: AnsiChar): PAnsiChar; cdecl;
begin
 Result := StrScan (s, c);
end;

function StrNCmp (const s1, s2: PAnsiChar; MaxLen: Cardinal): Integer; cdecl;
begin
 Result := StrLComp (s1, s2, MaxLen);
end;

end.

