unit rjPcreApi;

interface

const
 
 { Options -- Defined in pcre.h }
 
 PCRE_CASELESS      = $0001;
 PCRE_MULTILINE     = $0002;
 PCRE_DOTALL        = $0004;
 PCRE_EXTENDED      = $0008;
 PCRE_ANCHORED      = $0010;
 PCRE_DOLLAR_ENDONLY = $0020;
 PCRE_EXTRA         = $0040;
 PCRE_NOTBOL        = $0080;
 PCRE_NOTEOL        = $0100;
 PCRE_UNGREEDY      = $0200;
 PCRE_NOTEMPTY      = $0400;
 PCRE_UTF8          = $0800;
 
 { Exec-time and get-time error codes -- Defined in pcre.h }
 
 PCRE_ERROR_NOMATCH = -1;
 PCRE_ERROR_NULL    = -2;
 PCRE_ERROR_BADOPTION = -3;
 PCRE_ERROR_BADMAGIC = -4;
 PCRE_ERROR_UNKNOWN_NODE = -5;
 PCRE_ERROR_NOMEMORY = -6;
 PCRE_ERROR_NOSUBSTRING = -7;
 
 { Request types for pcre_fullinfo() -- Defined in pcre.h }
 
 PCRE_INFO_OPTIONS  = 0;
 PCRE_INFO_SIZE     = 1;
 PCRE_INFO_CAPTURECOUNT = 2;
 PCRE_INFO_BACKREFMAX = 3;
 PCRE_INFO_FIRSTCHAR = 4;
 PCRE_INFO_FIRSTTABLE = 5;
 PCRE_INFO_LASTLITERAL = 6;
 
type
 PInteger = ^Integer;
 PPAnsiChar = ^PAnsiChar;
 
 TPcre = Pointer;                       { Defined in pcre.h }
 TPcreExtra = Pointer;                  { Defined in pcre.h }
 
 { The real format of the start of the pcre block; the actual code vector
   runs on as long as necessary after the end. }
 
 PRealPcre = ^TRealPcre;
 TRealPcre = record                     { Defined in internals.h }
  magic_number: Cardinal;
  Size: Integer;
  tables: PChar;
  Options: Cardinal;
  top_bracket: Byte;
  top_backref: Byte;
  first_char: Byte;
  req_char: Byte;
  Code: PChar;
 end;
 
 { Functions -- Defined in pcre.h }
 
function pcre_compile (
 const Pattern: PAnsiChar;
 Options: Integer;
 const ErrPtr: PPAnsiChar;
 ErrOffset: PInteger;
 const TablePtr: PAnsiChar): TPcre;

function pcre_exec (
 const Code: TPcre;
 const Extra: TPcreExtra;
 const Subject: PAnsiChar;
 Length: Integer;
 StartOffSet: Integer;
 Options: Integer;
 OVector: PInteger; 
 OVecSize: Integer): Integer;

function pcre_fullinfo (
 const Code: TPcre;
 const Extra: TPcreExtra;
 What: Integer;
 Where: Pointer): Integer;

function pcre_info (
 const Code: TPcre;
 optptr: PInteger;
 firstcharptr: PInteger): Integer;

function pcre_maketables: PAnsiChar;

function pcre_study (
 const Code: TPcre;
 Options: Integer;
 ErrPtr: PPAnsiChar): TPcreExtra;

function pcre_version: PAnsiChar;

procedure pcre_dispose (var Code: TPcre; var Extra: TPcreExtra);

implementation

uses
 rjCHelpers;

{$L pcre.obj}
{$L study.obj}
{$L maketables.obj}

procedure pcre_dispose (var Code: TPcre; var Extra: TPcreExtra);
begin
 if (Code <> nil) then
  begin
   FreeMem (Code);
   Code := nil;
  end;
 if (Extra <> nil) then
  begin
   FreeMem (Extra);
   Extra := nil;
  end;
end;

{ **************************************************************************** }
{ External Declarations
{ **************************************************************************** }

function pcre_compile; external;
function pcre_exec; external;
function pcre_fullinfo; external;
function pcre_info; external;
function pcre_maketables; external;
function pcre_study; external;
function pcre_version; external;

end.

