{ Copyright Ralf Junker <ralfjunker@gmx.de> 2000
  http://www.zeitungsjunge.de/delphi/ }

unit rjPcreRegExp;

{$DEFINE rjPcreRegExpComponent}

interface

uses
 {$IFDEF rjPcreRegExpComponent}Classes, {$ENDIF}
 {$IFDEF RangeChecking}SysUtils, {$ENDIF}
 {$IFDEF CheckFree}rjCheckFree, {$ENDIF}
 rjPcreApi;

const
 { Set Default options here. Make sure they match your needs! }
 DEFAULT_AUTO_COMPILE = {$IFDEF rjPcreRegExpComponent}True{$ELSE}False{$ENDIF};
 DEFAULT_AUTO_STUDY = True;
 DEFAULT_COMPILE_OPTION_BITS = PCRE_CASELESS or PCRE_DOTALL;
 DEFAULT_MATCH_OPTION_BITS = 0;
 DEFAULT_CAPTURE_SUBSTRINGS = True;
 DEFAULT_USER_LOCALE = False;
 
type
 {$IFDEF RangeChecking}
 EPcreRegExp = Exception;
 {$ENDIF}
 
 PPcreVector = ^TPcreVector;
 TPcreVector = record
  FirstCharPos: Integer;
  AfterLastCharPos: Integer;
 end;
 
 PPcreVectorArray = ^TPcreVectorArray;
 TPcreVectorArray = array[0..MaxInt div SizeOf (TPcreVector) - 1] of TPcreVector;
 
 TCompileOption = (coAnchored, coCaseless, coDollarEndOnly, coDotAll, coExtended, coExtra, coMultiline, coUnGreedy, coUtf8);
 TCompileOptions = set of TCompileOption;
 
 TMatchOption = (moAnchored, moNotBol, moNotEol, moNotEmpty);
 TMatchOptions = set of TMatchOption;
 
 TPcreRegExp = class{$IFDEF rjPcreRegExpComponent} (TComponent){$ENDIF}
 private
  FCode: TPcre;                         { Compiled pattern. }
  FExtra: TPcreExtra;                   { Extra information to the compiled pattern. }
  FVector: PPcreVectorArray;            { Positions of substrings. }
  FVectorSize: Cardinal;                { Number of elements in FVector (Caution: Not its size in bytes!). }
  FTables: Pointer;                     { Holds the case tables. }

  FPattern: AnsiString;                 { Pattern itself. }

  FSubjectPointer: PAnsiChar;
  FSubjectLength: Cardinal;

  FSubStringCount: Integer;             { Number of substrings assigned during the last match. }

  FError: PChar;
  FErrorOffset: Integer;

  FCompileOptionBits: Cardinal;
  FMatchOptionBits: Cardinal;

  FAutoCompile: Boolean;
  FAutoStudy: Boolean;

  FCaptureSubstrings: Boolean;

  procedure SetPattern (const Pattern: AnsiString);

  function GetCompileOptions: TCompileOptions;
  procedure SetCompileOptions (const Value: TCompileOptions);
  procedure SetCompileOptionBits (const Value: Cardinal);

  function GetMatchOptions: TMatchOptions;
  procedure SetMatchOptions (const Value: TMatchOptions);

  function GetMatchedString: AnsiString;
  function GetMatchedLength: Cardinal;
  function GetMatchedFirstCharPos: Integer;
  function GetMatchedAfterLastCharPos: Integer;

  function GetSubString (const Index: Integer): AnsiString;
  function GetSubStringLength (const Index: Integer): Cardinal;
  function GetSubStringFirstCharPos (const Index: Integer): Integer;
  function GetSubStringAfterLastCharPos (const Index: Integer): Integer;

  procedure SetAutoCompile (const Value: Boolean);
  procedure SetAutoStudy (const Value: Boolean);

  function GetError: AnsiString;
  function GetUserLocale: Boolean;
  procedure SetUserLocale (const Value: Boolean);
  procedure SetCaptureSubstrings (const Value: Boolean);

 public
  constructor Create{$IFDEF rjPcreRegExpComponent} (AOwner: TComponent); override{$ENDIF};
  constructor CreateWithPattern ({$IFDEF rjPcreRegExpComponent}AOwner: TComponent; {$ENDIF}const Pattern: AnsiString); overload;

  destructor Destroy; override;

  procedure Clear;

  class function GetVersion: AnsiString;

  function Compile: Boolean; overload;
  function Compile (const Pattern: AnsiString): Boolean; overload;
  function Compile (const Pattern: AnsiString; const CompileOptionBits: Cardinal): Boolean; overload;

  function Study: Boolean;

  procedure SetSubjectString (const SubjectString: AnsiString);
  procedure SetSubjectBuffer (const SubjectBuffer; const SubjectBufferSize: Cardinal);

  function Match (const StartOffSet: Cardinal = 1): Integer; overload;
  function Match (const SubjectString: AnsiString; const StartOffSet: Cardinal = 1): Integer; overload;
  function Match (const Pattern, SubjectString: AnsiString; const StartOffSet: Cardinal = 1): Integer; overload;
  function Match (const SubjectBuffer: PAnsiChar; const SubjectBufferSize: Cardinal; const StartOffSet: Cardinal = 0): Integer; overload;

  property MatchedString: AnsiString read GetMatchedString;
  property MatchedLength: Cardinal read GetMatchedLength;
  property MatchedFirstCharPos: Integer read GetMatchedFirstCharPos;
  property MatchedAfterLastCharPos: Integer read GetMatchedAfterLastCharPos;

  property SubStringCount: Integer read FSubStringCount;
  property SubStrings[const Index: Integer]: AnsiString read GetSubString; default;
  property SubStringsLength[const Index: Integer]: Cardinal read GetSubStringLength;
  property SubStringsFirstCharPos[const Index: Integer]: Integer read GetSubStringFirstCharPos;
  property SubStringsAfterLastCharPos[const Index: Integer]: Integer read GetSubStringAfterLastCharPos;

  property Error: AnsiString read GetError;
 published
  property AutoCompile: Boolean read FAutoCompile write SetAutoCompile default DEFAULT_AUTO_COMPILE;
  property AutoStudy: Boolean read FAutoStudy write SetAutoStudy default DEFAULT_AUTO_STUDY;

  property CaptureSubstrings: Boolean read FCaptureSubstrings write SetCaptureSubstrings default DEFAULT_CAPTURE_SUBSTRINGS;

  property CompileOptions: TCompileOptions read GetCompileOptions write SetCompileOptions stored False;
  property MatchOptions: TMatchOptions read GetMatchOptions write SetMatchOptions stored False;

  property CompileOptionBits: Cardinal read FCompileOptionBits write SetCompileOptionBits default DEFAULT_COMPILE_OPTION_BITS;
  property MatchOptionBits: Cardinal read FMatchOptionBits write FMatchOptionBits default DEFAULT_MATCH_OPTION_BITS;

  property Pattern: AnsiString read FPattern write SetPattern;

  property UserLocale: Boolean read GetUserLocale write SetUserLocale default DEFAULT_USER_LOCALE;

  property Version: AnsiString read GetVersion;
 end;
 
function MatchPcreRegExpStr (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;
{ Returns >= 0 for Success, < 0 for Failure. }
function MatchPcreRegExpText (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;
{ Returns >= 0 for Success, < 0 for Failure. }

function PosPcreRegExpStr (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;

{ **************************************************************************** }

{$IFDEF rjPcreRegExpComponent}
procedure Register;
{$ENDIF}

implementation

uses
 {$IFDEF RangeChecking}
 Consts,
 {$ENDIF}
 
 SysUtils;

constructor TPcreRegExp.Create{$IFDEF rjPcreRegExpComponent} (AOwner: TComponent){$ENDIF};
begin
 {$IFDEF CheckFree}RegisterObjectCreate (Self); {$ENDIF}
 inherited Create{$IFDEF rjPcreRegExpComponent} (AOwner){$ENDIF};
 FAutoCompile := DEFAULT_AUTO_COMPILE;
 FAutoStudy := DEFAULT_AUTO_STUDY;
 FCompileOptionBits := DEFAULT_COMPILE_OPTION_BITS;
 FMatchOptionBits := DEFAULT_MATCH_OPTION_BITS;
 FCaptureSubstrings := DEFAULT_CAPTURE_SUBSTRINGS;
 UserLocale := DEFAULT_USER_LOCALE;
 FSubStringCount := -1;
end;

constructor TPcreRegExp.CreateWithPattern ({$IFDEF rjPcreRegExpComponent}AOwner: TComponent; {$ENDIF}const Pattern: AnsiString);
begin
 Create{$IFDEF rjPcreRegExpComponent} (AOwner){$ENDIF};
 FPattern := Pattern;
 if FAutoCompile then Compile;
end;

destructor TPcreRegExp.Destroy;
begin
 FreeMem (FCode);
 FreeMem (FExtra);
 FreeMem (FVector);
 FreeMem (FTables);
 inherited Destroy;
 {$IFDEF CheckFree}RegisterObjectFree (Self); {$ENDIF}
end;

procedure TPcreRegExp.Clear;
begin
 FreeMem (FCode);
 FCode := nil;
 FreeMem (FExtra);
 FExtra := nil;
 FreeMem (FVector);
 FVector := nil;
 FreeMem (FTables);
 FTables := nil;
 
 FPattern := '';
 FSubStringCount := -1;
end;

{ ********** }

function TPcreRegExp.Compile: Boolean;
label
 Fail;
begin
 FreeMem (FCode);
 FreeMem (FExtra);
 
 FSubStringCount := -1;
 
 FCode := pcre_compile (PChar (FPattern), FCompileOptionBits, @FError, @FErrorOffset, FTables);
 if FCode = nil then goto Fail;
 
 if FAutoStudy then
  begin
   FExtra := pcre_study (FCode, 0, @FError);
   if FError <> nil then goto Fail;
  end
 else
  FExtra := nil;
 
 if FCaptureSubstrings then
  begin
   { The following line corresponds to a call to the function in pcre.c:
     pcre_fullinfo (FCode, FExtra, PCRE_INFO_CAPTURECOUNT, @FVectorSize); }
   FVectorSize := PRealPcre (FCode)^.top_bracket;
   { PCRE Help: "The smallest size for ovector that will allow for
     n captured substrings in addition to the offsets of the substring
     matched by the whole pattern is (n+1)*3."
     We allocate double that amount to increase matching performance. }
   FVectorSize := (FVectorSize + 1) * 6;
   ReallocMem (FVector, FVectorSize * SizeOf (Cardinal));
  end
 else
  begin
   FVectorSize := 0;
   FreeMem (FVector);
   FVector := nil;
  end;
 
 Result := True;
 Exit;
 
 Fail:
 Result := False;
end;

function TPcreRegExp.Compile (const Pattern: AnsiString): Boolean;
begin
 FPattern := Pattern;
 Result := Compile;
end;

function TPcreRegExp.Compile (const Pattern: AnsiString; const CompileOptionBits: Cardinal): Boolean;
begin
 FPattern := Pattern;
 FCompileOptionBits := CompileOptionBits;
 Result := Compile;
end;

{ ********** }

function TPcreRegExp.Study: Boolean;
begin
 if FCode = nil then
  Result := False
 else
  begin
   FreeMem (FExtra);
   FExtra := pcre_study (FCode, 0, @FError);
   Result := FError = nil;
  end;
end;

{ ********** }

procedure TPcreRegExp.SetSubjectBuffer (const SubjectBuffer; const SubjectBufferSize: Cardinal);
begin
 FSubjectPointer := @SubjectBuffer;
 FSubjectLength := SubjectBufferSize;
 FSubStringCount := PCRE_ERROR_NOMATCH;
end;

procedure TPcreRegExp.SetSubjectString (const SubjectString: AnsiString);
begin
 if Pointer (SubjectString) <> nil then
  FSubjectLength := Cardinal (Pointer (Cardinal (SubjectString) - 4)^)
 else
  FSubjectLength := 0;
 FSubjectPointer := PChar (SubjectString);
 FSubStringCount := PCRE_ERROR_NOMATCH;
end;

{ ********** }

function TPcreRegExp.Match (const StartOffSet: Cardinal = 1): Integer;
begin
 Result := pcre_exec (FCode, FExtra, FSubjectPointer, FSubjectLength, StartOffSet - 1, FMatchOptionBits, Pointer (FVector), FVectorSize);
 FSubStringCount := Result;
end;

function TPcreRegExp.Match (const SubjectString: AnsiString; const StartOffSet: Cardinal = 1): Integer;
begin
 if Pointer (SubjectString) <> nil then
  FSubjectLength := Cardinal (Pointer (Cardinal (SubjectString) - 4)^)
 else
  FSubjectLength := 0;
 FSubjectPointer := PChar (SubjectString);
 Result := pcre_exec (FCode, FExtra, FSubjectPointer, FSubjectLength, StartOffSet - 1, FMatchOptionBits, Pointer (FVector), FVectorSize);
 FSubStringCount := Result;
end;

function TPcreRegExp.Match (const Pattern, SubjectString: AnsiString; const StartOffSet: Cardinal = 1): Integer;
begin
 if Pattern <> FPattern then
  begin
   FPattern := Pattern;
   Compile;
  end;
 if Pointer (SubjectString) <> nil then
  FSubjectLength := Cardinal (Pointer (Cardinal (SubjectString) - 4)^)
 else
  FSubjectLength := 0;
 FSubjectPointer := PChar (SubjectString);
 Result := pcre_exec (FCode, FExtra, FSubjectPointer, FSubjectLength, StartOffSet - 1, FMatchOptionBits, Pointer (FVector), FVectorSize);
 FSubStringCount := Result;
end;

{ ********** }

function TPcreRegExp.Match (const SubjectBuffer: PAnsiChar; const SubjectBufferSize: Cardinal; const StartOffSet: Cardinal = 0): Integer;
begin
 FSubjectPointer := SubjectBuffer;
 FSubjectLength := SubjectBufferSize;
 Result := pcre_exec (FCode, FExtra, FSubjectPointer, FSubjectLength, StartOffSet, FMatchOptionBits, Pointer (FVector), FVectorSize);
 FSubStringCount := Result;
end;

procedure TPcreRegExp.SetPattern (const Pattern: AnsiString);
begin
 if Pattern = FPattern then Exit;
 FPattern := Pattern;
 if FAutoCompile then
  Compile
 else
  begin
   FreeMem (FCode);
   FCode := nil;
   FreeMem (FExtra);
   FExtra := nil;
   FreeMem (FVector);
   FVector := nil;
   FSubStringCount := -1;
  end;
end;

{ ********** }

procedure TPcreRegExp.SetAutoCompile (const Value: Boolean);
begin
 if Value = FAutoCompile then Exit;
 FAutoCompile := Value;
 if FAutoCompile then Compile;
end;

procedure TPcreRegExp.SetAutoStudy (const Value: Boolean);
begin
 if Value = FAutoStudy then Exit;
 FAutoStudy := Value;
 if not Value then Exit;
 if FCode = nil then Exit;
 FreeMem (FExtra);
 FExtra := pcre_study (FCode, 0, @FError);
end;

procedure TPcreRegExp.SetCaptureSubstrings (const Value: Boolean);
begin
 if Value = FCaptureSubstrings then Exit;
 FCaptureSubstrings := Value;
 if FCode = nil then Exit;
 if Value then
  begin
   { The following line corresponds to a call to the function in pcre.c:
     pcre_fullinfo (FCode, FExtra, PCRE_INFO_CAPTURECOUNT, @FVectorSize); }
   FVectorSize := PRealPcre (FCode)^.top_bracket;
   { PCRE Help: "The smallest size for ovector that will allow for
     n captured substrings in addition to the offsets of the substring
     matched by the whole pattern is (n+1)*3."
     We allocate double that amount to increase matching performance. }
   FVectorSize := (FVectorSize + 1) * 6;
   ReallocMem (FVector, FVectorSize * SizeOf (Cardinal));
  end
 else
  begin
   FVectorSize := 0;
   FreeMem (FVector);
   FVector := nil;
  end;
end;

{ ********** }

class function TPcreRegExp.GetVersion: AnsiString;
begin
 Result := pcre_version;
end;

{ ********** }

function TPcreRegExp.GetCompileOptions: TCompileOptions;
begin
 Result := [];
 if (FCompileOptionBits and PCRE_CASELESS) <> 0 then Include (Result, coCaseless);
 if (FCompileOptionBits and PCRE_MULTILINE) <> 0 then Include (Result, coMultiline);
 if (FCompileOptionBits and PCRE_DOTALL) <> 0 then Include (Result, coDotAll);
 if (FCompileOptionBits and PCRE_EXTENDED) <> 0 then Include (Result, coExtended);
 if (FCompileOptionBits and PCRE_ANCHORED) <> 0 then Include (Result, coAnchored);
 if (FCompileOptionBits and PCRE_DOLLAR_ENDONLY) <> 0 then Include (Result, coDollarEndOnly);
 if (FCompileOptionBits and PCRE_EXTRA) <> 0 then Include (Result, coExtra);
 if (FCompileOptionBits and PCRE_UNGREEDY) <> 0 then Include (Result, coUnGreedy);
 if (FCompileOptionBits and PCRE_UTF8) <> 0 then Include (Result, coUtf8);
end;

procedure TPcreRegExp.SetCompileOptions (const Value: TCompileOptions);
begin
 FCompileOptionBits := 0;
 if coCaseless in Value then FCompileOptionBits := FCompileOptionBits or PCRE_CASELESS;
 if coMultiline in Value then FCompileOptionBits := FCompileOptionBits or PCRE_MULTILINE;
 if coDotAll in Value then FCompileOptionBits := FCompileOptionBits or PCRE_DOTALL;
 if coExtended in Value then FCompileOptionBits := FCompileOptionBits or PCRE_EXTENDED;
 if coAnchored in Value then FCompileOptionBits := FCompileOptionBits or PCRE_ANCHORED;
 if coDollarEndOnly in Value then FCompileOptionBits := FCompileOptionBits or PCRE_DOLLAR_ENDONLY;
 if coExtra in Value then FCompileOptionBits := FCompileOptionBits or PCRE_EXTRA;
 if coUnGreedy in Value then FCompileOptionBits := FCompileOptionBits or PCRE_UNGREEDY;
 if coUtf8 in Value then FCompileOptionBits := FCompileOptionBits or PCRE_UTF8;
 if FAutoCompile then Compile;
end;

procedure TPcreRegExp.SetCompileOptionBits (const Value: Cardinal);
begin
 if Value = FCompileOptionBits then Exit;
 FCompileOptionBits := Value;
 if FAutoCompile then Compile;
end;

{ ********** }

function TPcreRegExp.GetMatchOptions: TMatchOptions;
begin
 Result := [];
 if (FMatchOptionBits and PCRE_ANCHORED) <> 0 then Include (Result, moAnchored);
 if (FMatchOptionBits and PCRE_NOTBOL) <> 0 then Include (Result, moNotBol);
 if (FMatchOptionBits and PCRE_NOTEOL) <> 0 then Include (Result, moNotEol);
 if (FMatchOptionBits and PCRE_NOTEMPTY) <> 0 then Include (Result, moNotEmpty);
end;

procedure TPcreRegExp.SetMatchOptions (const Value: TMatchOptions);
begin
 FMatchOptionBits := 0;
 if moAnchored in Value then FMatchOptionBits := FMatchOptionBits or PCRE_ANCHORED;
 if moNotBol in Value then FMatchOptionBits := FMatchOptionBits or PCRE_NOTBOL;
 if moNotEol in Value then FMatchOptionBits := FMatchOptionBits or PCRE_NOTEOL;
 if moNotEmpty in Value then FMatchOptionBits := FMatchOptionBits or PCRE_NOTEMPTY;
end;

{ ********** }

function TPcreRegExp.GetError: AnsiString;
begin
 if FError = nil then
  Result := ''
 else
  Result := FError + ' (' + IntToStr (FErrorOffset) + ')';
end;

{ ********** }

function TPcreRegExp.GetMatchedString: AnsiString;
var
 l                  : Cardinal;
 pStart             : PChar;
begin
 {$IFDEF RangeChecking}
 if FSubStringCount <= 0 then raise EPcreRegExp.CreateFmt (SListIndexError, [0]);
 {$ENDIF}
 with FVector^[0] do
  begin
   pStart := FSubjectPointer + FirstCharPos;
   l := AfterLastCharPos - FirstCharPos;
  end;
 SetString (Result, pStart, l);
end;

function TPcreRegExp.GetMatchedLength: Cardinal;
begin
 {$IFDEF RangeChecking}
 if FSubStringCount <= 0 then raise EPcreRegExp.CreateFmt (SListIndexError, [0]);
 {$ENDIF}
 with FVector^[0] do
  Result := AfterLastCharPos - FirstCharPos;
end;

function TPcreRegExp.GetMatchedFirstCharPos: Integer;
begin
 {$IFDEF RangeChecking}
 if FSubStringCount <= 0 then raise EPcreRegExp.CreateFmt (SListIndexError, [0]);
 {$ENDIF}
 Result := FVector^[0].FirstCharPos
end;

function TPcreRegExp.GetMatchedAfterLastCharPos: Integer;
begin
 {$IFDEF RangeChecking}
 if FSubStringCount <= 0 then raise EPcreRegExp.CreateFmt (SListIndexError, [0]);
 {$ENDIF}
 Result := FVector^[0].AfterLastCharPos
end;

{ ********** }

function TPcreRegExp.GetSubString (const Index: Integer): AnsiString;
var
 l                  : Cardinal;
 pStart             : PChar;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FSubStringCount) then raise EPcreRegExp.CreateFmt (SListIndexError, [Index]);
 {$ENDIF}
 with FVector^[Index] do
  begin
   pStart := FSubjectPointer + FirstCharPos;
   l := AfterLastCharPos - FirstCharPos;
  end;
 SetString (Result, pStart, l);
end;

function TPcreRegExp.GetSubStringLength (const Index: Integer): Cardinal;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FSubStringCount) then raise EPcreRegExp.CreateFmt (SListIndexError, [Index]);
 {$ENDIF}
 with FVector^[Index] do
  Result := AfterLastCharPos - FirstCharPos;
end;

function TPcreRegExp.GetSubStringFirstCharPos (const Index: Integer): Integer;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FSubStringCount) then raise EPcreRegExp.CreateFmt (SListIndexError, [Index]);
 {$ENDIF}
 Result := FVector^[Index].FirstCharPos;
end;

function TPcreRegExp.GetSubStringAfterLastCharPos (const Index: Integer): Integer;
begin
 {$IFDEF RangeChecking}
 if (Index < 0) or (Index >= FSubStringCount) then raise EPcreRegExp.CreateFmt (SListIndexError, [Index]);
 {$ENDIF}
 Result := FVector^[Index].AfterLastCharPos;
end;

{ ********** }

function TPcreRegExp.GetUserLocale: Boolean;
begin
 Result := FTables <> nil;
end;

procedure TPcreRegExp.SetUserLocale (const Value: Boolean);
begin
 if Value = (FTables <> nil) then Exit;
 if Value then
  FTables := pcre_maketables
 else
  begin
   FreeMem (FTables);
   FTables := nil;
   Compile;
  end;
 if FAutoCompile then Compile;
end;

{ **************************************************************************** }

function MatchPcreRegExpStr (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;
label
 Fail, StudyFail;
var
 Code               : TPcre;
 Extra              : TPcreExtra;
 Error              : PChar;
 ErrOffset          : Integer;
begin
 Code := pcre_compile (PChar (Pattern), PCRE_MULTILINE or PCRE_ANCHORED, @Error, @ErrOffset, nil);
 if Code = nil then goto Fail;
 
 Extra := pcre_study (Code, 0, @Error);
 if Error <> nil then goto StudyFail;
 
 Result := pcre_exec (Code, Extra, PChar (Text), Length (Text), StartOffSet - 1, 0, nil, 0);
 
 FreeMem (Code);
 FreeMem (Extra);
 Exit;
 
 StudyFail:
 FreeMem (Code);
 Fail:
 Result := PCRE_ERROR_NOMATCH;
end;

function MatchPcreRegExpText (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;
label
 Fail, StudyFail;
var
 Code               : TPcre;
 Extra              : TPcreExtra;
 Error              : PChar;
 ErrOffset          : Integer;
begin
 Code := pcre_compile (PChar (Pattern), PCRE_CASELESS or PCRE_MULTILINE or PCRE_ANCHORED, @Error, @ErrOffset, nil);
 if Code = nil then goto Fail;
 
 Extra := pcre_study (Code, 0, @Error);
 if Error <> nil then goto StudyFail;
 
 Result := pcre_exec (Code, Extra, PChar (Text), Length (Text), StartOffSet - 1, 0, nil, 0);
 
 FreeMem (Code);
 FreeMem (Extra);
 Exit;
 
 StudyFail:
 FreeMem (Code);
 Fail:
 Result := PCRE_ERROR_NOMATCH;
end;

{ **************************************************************************** }

function PosPcreRegExpStr (const Pattern, Text: AnsiString; const StartOffSet: Cardinal = 1): Integer;
label
 Fail, StudyFail;
var
 Code               : TPcre;
 Extra              : TPcreExtra;
 Error              : PChar;
 ErrOffset          : Integer;
 Vector             : array[0..15] of Integer;
begin
 Code := pcre_compile (PChar (Pattern), PCRE_MULTILINE, @Error, @ErrOffset, nil);
 if Code = nil then goto Fail;
 
 Extra := pcre_study (Code, 0, @Error);
 if Error <> nil then goto StudyFail;
 
 if pcre_exec (Code, Extra, PChar (Text), Length (Text), StartOffSet - 1, 0, @Vector, SizeOf (Vector)) >= 0 then
  Result := Vector[0] + 1
 else
  Result := PCRE_ERROR_NOMATCH;
 
 FreeMem (Code);
 FreeMem (Extra);
 Exit;
 
 StudyFail:
 FreeMem (Code);
 Fail:
 Result := PCRE_ERROR_NOMATCH;
end;

{ **************************************************************************** }

{$IFDEF rjPcreRegExpComponent}
procedure Register;
begin
 RegisterComponents ('Standard', [TPcreRegExp]);
end;
{$ENDIF}

end.

